#include "ags_matrix.h"
#include "ags_matrix_callbacks.h"

#include "../ags_window.h"

#include "../../audio/ags_audio.h"
#include "../../audio/ags_channel.h"
#include "../../audio/ags_input.h"
#include "../../audio/ags_output.h"
#include "../../audio/ags_pattern.h"

#include <gtk/gtkframe.h>
#include <gtk/gtkvscrollbar.h>

GType ags_matrix_get_type(void);
void ags_matrix_class_init(AgsMatrixClass *matrix);
void ags_matrix_init(AgsMatrix *matrix);
void ags_matrix_connect(AgsMatrix *matrix);
void ags_matrix_destroy(GtkObject *object);
void ags_matrix_show(GtkWidget *widget);
void ags_matrix_set_audio_channels(AgsAudio *audio, guint audio_channels);
void ags_matrix_set_pads(AgsAudio *audio, GType type, guint pads);
void ags_matrix_set_lines(AgsAudio *audio, GType type, guint lines);

extern void ags_file_read_matrix(AgsFile *file, AgsMachine *machine);
extern void ags_file_write_matrix(AgsFile *file, AgsMachine *machine);

GtkStyle *matrix_style;

#define AGS_MATRIX_CELL_WIDTH   12
#define AGS_MATRIX_CELL_HEIGHT  10

GType
ags_matrix_get_type(void)
{
  static GType matrix_type = 0;

  if (!matrix_type){
    static const GtkTypeInfo matrix_info = {
      "AgsMatrix\0",
      sizeof(AgsMatrix), /* base_init */
      sizeof(AgsMatrixClass), /* base_finalize */
      (GtkClassInitFunc) ags_matrix_class_init,
      (GtkObjectInitFunc) ags_matrix_init,
      NULL, /* class_finalize */
      NULL, /* class_data */
      (GtkClassInitFunc) NULL,
    };

    matrix_type = gtk_type_unique (AGS_TYPE_MACHINE, &matrix_info);
  }

  return (matrix_type);
}

void
ags_matrix_class_init(AgsMatrixClass *matrix)
{
  GtkObjectClass *object = (GtkObjectClass *) matrix;
  GtkWidgetClass *widget = (GtkWidgetClass *) matrix;
  AgsMachineClass *machine = (AgsMachineClass *) matrix;

  object->destroy = ags_matrix_destroy;

  widget->show = ags_matrix_show;

  machine->read_file = ags_file_read_matrix;
  machine->write_file = ags_file_write_matrix;
}

void
ags_matrix_init(AgsMatrix *matrix)
{
  AgsChannel *channel;
  GtkFrame *frame;
  GtkTable *table;
  GtkToggleButton *button;
  GtkVScrollbar *vscrollbar;
  int i, j;

  frame = (GtkFrame *) (gtk_container_get_children((GtkContainer *) matrix))->data;

  matrix->table = (GtkTable *) gtk_table_new(2, 3, FALSE);
  gtk_container_add((GtkContainer *) frame, (GtkWidget *) matrix->table);

  table = (GtkTable *) gtk_table_new(3, 3, FALSE);
  gtk_table_attach(matrix->table, (GtkWidget *) table,
		   0, 1, 0, 1,
		   GTK_EXPAND, GTK_EXPAND,
		   0, 0);
  matrix->selected = NULL;

  for(i = 0; i < 3; i++){
    for(j = 0; j < 3; j++){
      matrix->index[i * 3 +j] =
	button = (GtkToggleButton *) gtk_toggle_button_new_with_label(g_strdup_printf("%d\0", i * 3 + j +1));
      gtk_table_attach(table, (GtkWidget *) button,
		       j, j +1, i, i +1,
		       GTK_EXPAND, GTK_EXPAND,
		       0, 0);
    }
  }

  matrix->selected = matrix->index[0];
  gtk_toggle_button_set_active(matrix->selected, TRUE);

  table = (GtkTable *) gtk_table_new(2, 2, FALSE);
  gtk_table_attach(matrix->table, (GtkWidget *) table,
		   1, 2, 0, 1,
		   GTK_EXPAND, GTK_EXPAND,
		   0, 0);

  matrix->drawing_area = (GtkDrawingArea *) gtk_drawing_area_new();
  gtk_widget_set_size_request((GtkWidget *) matrix->drawing_area, 32 * AGS_MATRIX_CELL_WIDTH +1, 8 * AGS_MATRIX_CELL_HEIGHT +1);
  gtk_widget_set_style((GtkWidget *) matrix->drawing_area, matrix_style);
  gtk_table_attach(table, (GtkWidget *) matrix->drawing_area,
		   0, 1, 0, 1,
		   GTK_EXPAND, GTK_EXPAND,
		   0, 0);
  gtk_widget_set_events ((GtkWidget *) matrix->drawing_area,
                         GDK_EXPOSURE_MASK
                         | GDK_LEAVE_NOTIFY_MASK
                         | GDK_BUTTON_PRESS_MASK
                         | GDK_POINTER_MOTION_MASK
                         | GDK_POINTER_MOTION_HINT_MASK);


  matrix->adjustment = (GtkAdjustment *) gtk_adjustment_new(0.0, 0.0, 0.0, 1.0, 1.0, 8.0);

  vscrollbar = (GtkVScrollbar *) gtk_vscrollbar_new(matrix->adjustment);
  gtk_widget_set_style((GtkWidget *) vscrollbar, matrix_style);
  gtk_table_attach(table, (GtkWidget *) vscrollbar,
		   1, 2, 0, 1,
		   GTK_EXPAND | GTK_FILL, GTK_EXPAND | GTK_FILL,
		   0, 0);

  AGS_AUDIO_GET_CLASS(matrix->machine.audio)->set_audio_channels = ags_matrix_set_audio_channels;
  AGS_AUDIO_GET_CLASS(matrix->machine.audio)->set_pads = ags_matrix_set_pads;
  AGS_AUDIO_GET_CLASS(matrix->machine.audio)->set_lines = ags_matrix_set_lines;

  matrix->machine.audio->audio_channels = 1;
  ags_audio_set_pads(matrix->machine.audio, AGS_TYPE_OUTPUT, 1);
  ags_audio_set_pads(matrix->machine.audio, AGS_TYPE_INPUT, 64);
}

void
ags_matrix_destroy(GtkObject *object)
{
  ags_machine_destroy(object);
}

void
ags_matrix_show(GtkWidget *widget)
{
  AgsMatrix *matrix = (AgsMatrix *) widget;
  AgsWindow *window;

  window = (AgsWindow *) gtk_widget_get_toplevel(widget);

  if(matrix->machine.name == NULL)
    matrix->machine.name = g_strdup_printf("Default %d\0", window->counter->matrix);

  window->counter->matrix++;

  ags_machine_show(widget);

  gtk_widget_show_all((GtkWidget *) matrix->table);
}

void
ags_matrix_connect(AgsMatrix *matrix)
{
  int i;

  for(i  = 0; i < 9; i++){
    g_signal_connect (G_OBJECT (matrix->index[i]), "clicked\0",
		      G_CALLBACK (ags_matrix_index_callback), (gpointer) matrix);
  }

  g_signal_connect (G_OBJECT (matrix->drawing_area), "expose_event\0",
                    G_CALLBACK (ags_matrix_drawing_area_expose_callback), (gpointer) matrix);

  g_signal_connect (G_OBJECT (matrix->drawing_area), "button_press_event\0",
                    G_CALLBACK (ags_matrix_drawing_area_button_press_callback), (gpointer) matrix);

  g_signal_connect (G_OBJECT (matrix->adjustment), "value_changed\0",
                    G_CALLBACK (ags_matrix_adjustment_value_changed_callback), (gpointer) matrix);
}

void
ags_matrix_set_audio_channels(AgsAudio *audio, guint audio_channels)
{
  printf("AgsMatrix only pads can be adjusted\n\0");
  //  _ags_audio_set_audio_channels(audio, audio_channels);
}

void
ags_matrix_set_pads(AgsAudio *audio, GType type, guint pads)
{
  guint old_pads;

  old_pads = (type == AGS_TYPE_OUTPUT) ? audio->output_pads: audio->input_pads;

  if(pads == old_pads)
    return;

  if(pads < 8){
    printf("AgsMatrix minimum pad count 8\n\0");
    pads = 8;
  }

  _ags_audio_set_pads(audio, type, pads);

  if(type == AGS_TYPE_INPUT)
    if(old_pads < pads){
      AgsChannel *channel;

      channel = ags_channel_nth(audio->input, old_pads);

      while(channel != NULL){
	channel->pattern = g_list_alloc();
	channel->pattern->data = (gpointer) ags_pattern_new();
	ags_pattern_set_dim((AgsPattern *) channel->pattern->data, 1, 9, 32);

	channel = channel->next;
      }
    }else{
    }
}

void
ags_matrix_set_lines(AgsAudio *audio, GType type, guint lines)
{
  _ags_audio_set_lines(audio, type, lines);
}

AgsMatrix*
ags_matrix_new()
{
  AgsMatrix *matrix;

  matrix = (AgsMatrix *) g_object_new(AGS_TYPE_MATRIX, NULL);

  return(matrix);
}
